"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const emotion_1 = require("@leafygreen-ui/emotion");
const hadron_document_1 = require("hadron-document");
const auto_focus_context_1 = require("./auto-focus-context");
const use_force_update_1 = require("./use-force-update");
const element_1 = require("./element");
const use_previous_1 = require("./use-previous");
const visible_field_toggle_1 = __importDefault(require("./visible-field-toggle"));
const typography_1 = require("./typography");
function useHadronDocument(doc) {
    const prevDoc = (0, use_previous_1.usePrevious)(doc);
    const forceUpdate = (0, use_force_update_1.useForceUpdate)();
    const onVisibleElementsChanged = (0, react_1.useCallback)((document) => {
        if (document === doc) {
            forceUpdate();
        }
    }, [doc, forceUpdate]);
    const onDocumentFieldsAddedOrRemoved = (0, react_1.useCallback)((_el, parentEl) => {
        // We only care about elements added or removed to the root document here
        // so that the root elements list can be correctly re-rendered. Everything
        // else will be handled correctly by HadronElement component
        if (doc === parentEl) {
            forceUpdate();
        }
    }, [doc, forceUpdate]);
    (0, react_1.useEffect)(() => {
        // Force update if the document that was passed to the component changed
        if (prevDoc && prevDoc !== doc) {
            forceUpdate();
        }
    }, [prevDoc, doc, forceUpdate]);
    (0, react_1.useEffect)(() => {
        doc.on(hadron_document_1.DocumentEvents.VisibleElementsChanged, onVisibleElementsChanged);
        doc.on(hadron_document_1.ElementEvents.Added, onDocumentFieldsAddedOrRemoved);
        doc.on(hadron_document_1.ElementEvents.Removed, onDocumentFieldsAddedOrRemoved);
        return () => {
            doc.off(hadron_document_1.DocumentEvents.VisibleElementsChanged, onVisibleElementsChanged);
            doc.off(hadron_document_1.ElementEvents.Added, onDocumentFieldsAddedOrRemoved);
            doc.off(hadron_document_1.ElementEvents.Removed, onDocumentFieldsAddedOrRemoved);
        };
    }, [doc, onDocumentFieldsAddedOrRemoved, onVisibleElementsChanged]);
    return {
        elements: [...doc.elements],
        visibleElements: doc.getVisibleElements(),
    };
}
const hadronDocument = (0, emotion_1.css)({
    position: 'relative',
    fontFamily: typography_1.documentTypography.fontFamily,
    fontSize: `${typography_1.documentTypography.fontSize}px`,
    lineHeight: `${typography_1.documentTypography.lineHeight}px`,
    counterReset: 'line-number',
});
// TODO: This element should implement treegrid aria role to be accessible
// https://www.w3.org/TR/wai-aria-practices/examples/treegrid/treegrid-1.html
// https://jira.mongodb.org/browse/COMPASS-5614
const HadronDocument = ({ value: document, editable = false, editing = false, onEditStart, extraGutterWidth, }) => {
    const { elements, visibleElements } = useHadronDocument(document);
    const [autoFocus, setAutoFocus] = (0, react_1.useState)(null);
    (0, react_1.useEffect)(() => {
        if (!editing) {
            setAutoFocus(null);
        }
    }, [editing]);
    const handleVisibleFieldsChanged = (0, react_1.useCallback)((totalVisibleFields) => {
        document.setMaxVisibleElementsCount(totalVisibleFields);
    }, [document]);
    // To render the "Show more" toggle for the document we need to calculate a
    // proper offset so that it aligns with the expand icon of top level fields
    const showMoreToggleOffset = (0, react_1.useMemo)(() => (0, element_1.calculateShowMoreToggleOffset)({
        editable,
        level: 0,
        alignWithNestedExpandIcon: false,
        extraGutterWidth,
    }), [editable, extraGutterWidth]);
    return (react_1.default.createElement("div", null,
        react_1.default.createElement("div", { className: hadronDocument, "data-testid": "hadron-document", "data-id": document.uuid },
            react_1.default.createElement(auto_focus_context_1.AutoFocusContext.Provider, { value: autoFocus }, visibleElements.map((el, idx) => {
                return (react_1.default.createElement(element_1.HadronElement, { key: idx, value: el, editable: editable, editingEnabled: editing, onEditStart: editable
                        ? (id, type) => {
                            setAutoFocus({ id, type });
                            onEditStart?.();
                        }
                        : undefined, lineNumberSize: visibleElements.length, onAddElement: (el) => {
                        setAutoFocus({
                            id: el.uuid,
                            type: el.parent?.currentType === 'Array' ? 'value' : 'key',
                        });
                    }, extraGutterWidth: extraGutterWidth }));
            }))),
        react_1.default.createElement(visible_field_toggle_1.default
        // TODO: "Hide items" button will only be shown when document is not
        // edited because it's not decided how to handle changes to the fields
        // that are changed but then hidden
        // https://jira.mongodb.org/browse/COMPASS-5587
        , { 
            // TODO: "Hide items" button will only be shown when document is not
            // edited because it's not decided how to handle changes to the fields
            // that are changed but then hidden
            // https://jira.mongodb.org/browse/COMPASS-5587
            showHideButton: !editing, currentSize: document.maxVisibleElementsCount, totalSize: elements.length, minSize: hadron_document_1.DEFAULT_VISIBLE_DOCUMENT_ELEMENTS, 
            // In the editing mode we allow to show / hide less fields because
            // historically Compass was doing this for "performance" reasons
            step: editing ? 100 : 1000, onSizeChange: handleVisibleFieldsChanged, style: {
                paddingLeft: showMoreToggleOffset,
            } })));
};
exports.default = HadronDocument;
//# sourceMappingURL=document.js.map